%	function [memb, R, vals]=JMFuzzyK(spiketimes,nclusters,Gwidth,squash,fuzzyfactor,mode)
%	
%		spiketimes: data: Ntrials x Nspikes (0 for no spikes)
%		nclusters: Number of clusters
%		Gwidth: Smoothing gaussian (s) - use 1e-3
%		squash: introduction of correlation non-linearity (0 for none, use 0.05)
%		fuzzyfactor: between 1 and 3 (use 2 or slightly less)
%		mode: graphical output=1
%
%		R: re-ordered trials
%		memb: membership array
%       vals: return useful algorithm-dependent arrays (debugging)
% record the fuzzyfactors and squash that yield far apart centers
%
%	Author: Jean-Marc Fellous
%			fellous@salk.edu


function [memb, R, vals]=JMFuzzyK(rspiketimes, nclusters, sigma, squash, fuzzyfactor, mode,iniT,endT)

global maxT nwaves waves		% maxT=length of a trial (seconds)

DEBUG=1;
Nini=1;

ntrials=size(rspiketimes,1);
nwvs=size(rspiketimes,2);
maxnspikes=size(rspiketimes,3);
npoints=size(waves,1);
srate=npoints/maxT;
symb='ox>*sdv^<+ph.';
cols='rybgwmcrybgwm';

l=4*sigma;					%length of convolution window
t=linspace(0,l,2*ceil(l*srate/2));		% make sure it is even
kernel=exp(-((t-l/2).^2)/(2*sigma*sigma));	% gaussian
	
stimes=zeros(ntrials,1,maxnspikes);

R=zeros(ntrials,nwvs,maxnspikes);
memb=cell(1,nwvs);

vals=cell(3,nwvs);

for w=1:nwvs
	
	buf=sprintf('--> FuzzyKMeanUnix-Processing wave %d/%d',w,nwvs);
	disp(buf)
	
	stimes(:,1,:)=rspiketimes(:,w,:);		

	[r2, Allc]= corr_relf(stimes,sigma,0,iniT,endT,1);
	
	Alls=Allc + Allc' - eye(size(Allc));		% symmetric matrix
    
    	% Introduce non-linearity
	
	corvals=[];
	for i=1:size(Alls,1)-1
		corvals=[corvals, Alls(i,i+1:size(Alls,1))];
    end
	
	id=find(corvals>0);		    
	normAlls=Alls/(2*mean(corvals(id)));
	g=find(normAlls>1);normAlls(g)=1;
		
    if (squash>0)
    	mins=Inf;Alls2=normAlls;
    	%figure(3);clf;figure(4);clf;
    	for sqsh=0.01:0.005:0.3
			Alls2b=1./(1+exp(-(normAlls-0.5)/sqsh));	% smaller the dividing factor, the higher the slope
			corvals2b=[];
			for i=1:size(Alls2b,1)-1
				corvals2b=[corvals2b, Alls2b(i,i+1:size(Alls2b,1))];
			end
			[N2 X2]=hist(corvals2b(:),linspace(0,1,50));
			s=std(N2);
			%figure(3);bar(X2,N2);ylabel(sprintf('squash=%.3f',sqsh));set(gca,'XLim',[0 1]);set(gca,'YLim',[0 1500]);drawnow;pause;
			%figure(4);hold on; plot(sqsh,s,'o');
			if s<mins & (N2(2)~=0)
				mins=s;
				Alls2=Alls2b;
				squash=sqsh;
			end
			if (N2(2)==0)		% squash too shallow (N2(1)==0 should be the minimum)
				break
			end
        end
	
		method=sprintf('squashed %.2f and renormalized',squash);
		%figure(3);plot(0.01:0.01:0.3,vals);
	elseif (squash==0)
		Alls2=normAlls;  method='renormalized, no squash'; % renormalization, no squash  
	else	% squash <0
		Alls2=Alls;  method='no renormalization, no squash'; % no renormalization, no squash
	end
	
    mins=Inf;
	mxs=-Inf;
	
	for ini=1:Nini	% loop on several initial conditions	
			
		% iterate until the centers are far enough. decrease the
		% fuzzyfactor at each iteration (adaptive method)
		fuzstep=0.05;fuzfact=fuzzyfactor;
		for fuzfact=fuzzyfactor:-fuzstep:1+fuzstep
			[clss,u,ctrs,err, Nbiterations]=dcFuzzy(Alls2,nclusters,fuzfact);
			p=1;j=1;
			for k=1:nclusters-1
				for kk=k+1:nclusters
					%p=p*mean(abs(center(k,:)-center(kk,:)));
					p=p*distfcm(ctrs(k,:),ctrs(kk,:));
				end
				j=j+1;
			end
			if(p>10^(-j)& Nbiterations<300)
				break	% centers are far enough
			end
			%fuzfact=sqrt(fuzfact);		% square-root strategy
			%fuzfact=fuzfact-0.05;		% linear strategy
			
			%buf=sprintf('Centers too close, iteration (%d/%d) : Fuzz=%3.2f',ini, Nini,fuzfact);
			%disp(buf)
		end	
		if(fuzfact ~=fuzzyfactor)
			buf=sprintf('Automatic fuzz adjustment, iteration (%d/%d) : Fuzz=%3.2f',ini, Nini,fuzfact);
			disp(buf)
		end
			
		
	end
	
	
	Classes=clss;
	vals(1,w)={squash};
	vals(2,w)={[fuzfact Nbiterations]};
	
    	idx=cell(1,nclusters);
    	indx=[];
    	for k=1:nclusters
	    idx{k}=find(Classes(:)==k);
	    indx=[indx;idx{k}];
    	end
	
	ospiketimes=rspiketimes(indx,w,:);
	memb(1,w)={idx};			% sets the output values
	R(:,w,:)=ospiketimes;
	
	d=distfcm(ctrs,Alls2);			% compute and return cluster distances
	minD = min(d);
	clustdist=zeros(2,nclusters);
	for k=1:nclusters
		a=idx(1,k);ink=a{1};outk=setdiff(1:ntrials,ink);
		%if(mean(d(k,ink))>0 & ~isempty(outk))
		if(~isempty(ink) & ~isempty(outk))
			clustdist(1,k)=mean(d(k,ink));                  % average of the distance from center to all the points within the cluster
			clustdist(2,k)=mean(d(k,outk));                  % average of the distance from center to all the points not belonging to the cluster 
		end
	end
	vals(3,w)={clustdist};
	
    
    % Do the plotting............
    
	if (mode>0)
	    h=figure(1);
	    set(h,'Position',[20 300 400 600])
	    
	    clf 
	
	    subplot(2,1,1)
	    rasterplot(stimes,1,0); 
	    shift=(endT-iniT)/100;	% places the symbols before iniT, 100th of the distance between iniT and endT
	    for k=1:nclusters
		    a=idx(1,k);
		    if length(a{1})>0
		    	plot(-k*shift+iniT,1+a{1},strcat(symb(k),cols(k)))
		    end
	    end
	    set(gca,'XLim',[-nclusters*shift+iniT, endT])
	    subplot(2,1,2)
	    
	    rasterplot(ospiketimes,1,0); 
	    i=0;
	    for k=1:nclusters
		    a=idx(1,k);
		    if length(a{1})>0
		    	plot(-k*shift+iniT,i+1:i+length(a{1}),strcat(symb(k),cols(k)))
		    end
		    i=i+length(a{1});
	    end
	    set(gca,'XLim',[-nclusters*shift+iniT, endT])
	    
	    drawnow
	    
	    h=figure(2);
	    set(h,'Position',[1010 300 250 600])
	    clf
	    greycols(255)
	    
	    subplot(2,1,1)
	    imagesc(Alls2)
	    hold on
	    for k=1:nclusters
		    a=idx(1,k);
		    if length(a{1})>0
		    	plot(-nclusters-1+k,a{1},strcat(symb(k),cols(k)))
		    end
	    end
	    set(gca,'XLim',[-nclusters-2, size(Alls2,1)])
	    axis('image')
	    title(method)
	    
	    subplot(2,1,2)
	    imagesc(Alls2(indx,indx))
	    hold on
	    i=0;
	
	    for k=1:nclusters
		    a=idx(1,k);
		    if length(a{1})>0
		    	plot(-nclusters-1+k,i+1:i+length(a{1}),strcat(symb(k),cols(k)));
		    end
		    i=i+length(a{1});
	    end
	    set(gca,'XLim',[-nclusters-2, size(Alls2,1)])
	    axis('image')
	    drawnow
	end	
end	
	
% --------- debug zone	----------------














